@@ -1,5 +1,6 @@
 # Function calls
 charToFact  = function(x, decreasing=FALSE, addNA=TRUE, nThread=getOption("kit.nThread")) .Call(CcharToFactR, x, decreasing, nThread, NA, parent.frame(), addNA)
+clearData   = function(x, verbose=FALSE) .Call("CclearMappingObjectR", x, verbose)
 count       = function(x, value) .Call(CcountR, x, value)
 countNA     = function(x) .Call(CcountNAR, x)
 countOccur  = function(x) .Call(CcountOccurR, x)
@@ -10,7 +11,14 @@ iif         = function(test, yes, no, na=NULL, tprom=FALSE, nThread=getOption("k
 nif         = function(..., default=NULL) .Call(CnifR, default, parent.frame(), as.list(substitute(...())))
 nswitch     = function(x, ..., default=NULL, nThread=getOption("kit.nThread"), checkEnc = TRUE) .Call(CnswitchR, x, default, nThread, checkEnc, list(...))
 pall        = function(..., na.rm=FALSE) .Call(CpallR, na.rm, if (...length() == 1L && is.list(..1)) ..1 else list(...))
+pallNA      = function(...) { x = if (...length() == 1L && is.list(..1)) unclass(..1) else list(...); .Call(CpcountNAR, x) == length(x) }
+pallv       = function(..., value) { x = if (...length() == 1L && is.list(..1)) unclass(..1) else list(...); pcount(..., value = value) == length(x) }                                                                     
 pany        = function(..., na.rm=FALSE) .Call(CpanyR, na.rm, if (...length() == 1L && is.list(..1)) ..1 else list(...))
+panyNA      = function(...) as.logical(pcountNA(...))
+panyv       = function(..., value) as.logical(pcount(..., value = value))
+pcountNA    = function(...) .Call(CpcountNAR, if (...length() == 1L && is.list(..1)) ..1 else list(...))
+pfirst      = function(...) .Call(CpfirstR, FALSE, if (...length() == 1L && is.list(..1)) ..1 else list(...))
+plast       = function(...) .Call(CpfirstR, TRUE, if (...length() == 1L && is.list(..1)) ..1 else list(...))
 pmean       = function(..., na.rm=FALSE) .Call(CpmeanR, na.rm, if (...length() == 1L && is.list(..1)) ..1 else list(...))
 pprod       = function(..., na.rm=FALSE) .Call(CpprodR, na.rm, if (...length() == 1L && is.list(..1)) ..1 else list(...))
 psum        = function(..., na.rm=FALSE) .Call(CpsumR,  na.rm, if (...length() == 1L && is.list(..1)) ..1 else list(...))
@@ -19,13 +27,13 @@ topn        = function(vec, n=6L, decreasing=TRUE, hasna=TRUE,index=TRUE) if(ind
 uniqLen     = function(x) .Call(CdupLenR, x)
 vswitch     = function(x, values, outputs, default=NULL, nThread=getOption("kit.nThread"), checkEnc = TRUE) .Call(CvswitchR, x, values, outputs, default, nThread, checkEnc)
 
-.onAttach   = function(libname, pkgname) packageStartupMessage(paste0("Attaching kit 0.0.11 (OPENMP ",if(.Call(CompEnabledR)) "enabled" else "disabled"," using 1 thread)"))
+.onAttach   = function(libname, pkgname) packageStartupMessage(paste0("Attaching kit 0.0.12 (OPENMP ",if(.Call(CompEnabledR)) "enabled" else "disabled"," using 1 thread)"))
 .onLoad     = function(libname, pkgname) options("kit.nThread"=1L)   #nocov
 .onUnload   = function(libpath) library.dynam.unload("kit", libpath) #nocov
 
 pcount      = function(..., value) {
   if(is.na(value[1])) {
-    .Call(CpcountNAR, value, if (...length() == 1L && is.list(..1)) ..1 else list(...))
+    .Call(CpcountNAR, if (...length() == 1L && is.list(..1)) ..1 else list(...))
   } else {
     .Call(CpcountR, value, if (...length() == 1L && is.list(..1)) ..1 else list(...))
   }
@@ -39,8 +47,6 @@ psort = function(x, decreasing = FALSE, na.last = NA, nThread=getOption("kit.nTh
   sort(x, decreasing = decreasing, na.last = na.last,method = if(c.locale) "radix" else "quick")
 }
 
-clearData = function(x, verbose=FALSE) .Call("CclearMappingObjectR", x, verbose)
-
 shareData = function(data, map_name, verbose=FALSE) {
   conn = rawConnection(raw(0L), "w")
   serialize(data, conn)
