@@ -31,16 +31,16 @@ SEXP psumR(SEXP na, SEXP args) {
   SEXPTYPE type0 = anstype;
   const R_xlen_t len0 = xlength(args0);
   Rboolean hasFactor = isFactor(args0);
-  if (anstype != INTSXP && anstype != REALSXP && anstype != CPLXSXP) {
-    error("Argument %d is of type %s. Only integer, double and complex types are supported."
-          "Data.frame (of the previous types) is also supported as a single input. ",
+  if (anstype != LGLSXP && anstype != INTSXP && anstype != REALSXP && anstype != CPLXSXP) {
+    error("Argument %d is of type %s. Only integer/logical, double and complex types are supported. "
+          "A data.frame (of the previous types) is also supported as a single input. ",
           1, type2char(anstype));
   }
   for (int i = 1; i < n; ++i) {
     SEXPTYPE type = UTYPEOF(PTR_ETL(args, i));
     R_xlen_t len1 = xlength(PTR_ETL(args, i));
-    if (type != INTSXP && type != REALSXP && type != CPLXSXP) {
-      error("Argument %d is of type %s. Only integer, double and complex types are supported. " , i+1, type2char(type));
+    if (type != LGLSXP && type != INTSXP && type != REALSXP && type != CPLXSXP) {
+      error("Argument %d is of type %s. Only integer/logical, double and complex types are supported. " , i+1, type2char(type));
     }
     if (len1 != len0) {
       error("Argument %d is of length %zu but argument %d is of length %zu. "
@@ -52,6 +52,7 @@ SEXP psumR(SEXP na, SEXP args) {
     }
     hasFactor = hasFactor ? TRUE : isFactor(PTR_ETL(args, i));
   }
+  if(anstype == LGLSXP) anstype = INTSXP; // We can sum logical vectors into an integer vector
   if (hasFactor) {
     error("Function 'psum' is not meaningful for factors.");
   }
@@ -164,16 +165,16 @@ SEXP pprodR(SEXP na, SEXP args) {
   SEXPTYPE type0 = anstype;
   const R_xlen_t len0 = xlength(args0);
   Rboolean hasFactor = isFactor(args0);
-  if (anstype != INTSXP && anstype != REALSXP && anstype != CPLXSXP) {
-    error("Argument %d is of type %s. Only integer, double and complex types are supported."
-          "Data.frame (of the previous types) is also supported as a single input. ",
+  if (anstype != LGLSXP && anstype != INTSXP && anstype != REALSXP && anstype != CPLXSXP) {
+    error("Argument %d is of type %s. Only integer/logical, double and complex types are supported. "
+          "A data.frame (of the previous types) is also supported as a single input. ",
           1, type2char(anstype));
   }
   for (int i = 1; i < n; ++i) {
     SEXPTYPE type = UTYPEOF(PTR_ETL(args, i));
     R_xlen_t len1 = xlength(PTR_ETL(args, i));
-    if (type != INTSXP && type != REALSXP && type != CPLXSXP) {
-      error("Argument %d is of type %s. Only integer, double and complex types are supported.", i+1, type2char(type));
+    if (type != LGLSXP && type != INTSXP && type != REALSXP && type != CPLXSXP) {
+      error("Argument %d is of type %s. Only integer/logical, double and complex types are supported.", i+1, type2char(type));
     }
     if (len1 != len0) {
       error("Argument %d is of length %zu but argument %d is of length %zu. "
@@ -192,7 +193,9 @@ SEXP pprodR(SEXP na, SEXP args) {
   SEXP ans = anstype != type0 ? PROTECT(coerceVector(args0, anstype)) : PROTECT(duplicate(args0));
   const bool narm = asLogical(na);
   switch(anstype) {
-  case INTSXP: {
+  case LGLSXP:   // This is useful, pprod can basically work like pall() for logical arguments, and also works if some arguments are not logical..
+  case INTSXP: { // Multiplication will likely cause integer overflows. So likely that base::prod only returns doubles, see typeof(base::prod(1:4))...
+                 // So the question is should there be integer return at all, or rather double return if all inputs are integer or logical..
     int *restrict pans =INTEGER(ans);
     if(narm) {
       for (ssize_t j = 0; j < len0; ++j) {
@@ -213,6 +216,7 @@ SEXP pprodR(SEXP na, SEXP args) {
         }
       }
     }
+    // if(anstype == LGLSXP) SET_TYPEOF(ans, LGLSXP); // Not needed, can use pall() instead if we want this...
   } break;
   case REALSXP: {
     double *restrict pans = REAL(ans);
@@ -409,20 +413,19 @@ SEXP pmeanR(SEXP na, SEXP args) {
     error("Please supply at least 1 argument. (%d argument supplied)", n);
   }
   const SEXP args0 = PTR_ETL(args, 0);
-  SEXPTYPE anstype = UTYPEOF(args0);
-  SEXPTYPE type0 = REALSXP;
+  SEXPTYPE type0 = UTYPEOF(args0);
   const R_xlen_t len0 = xlength(args0);
   Rboolean hasFactor = isFactor(args0);
-  if (anstype != INTSXP && anstype != REALSXP) {
-    error("Argument %d is of type %s. Only integer and double types are supported."
-          "Data.frame (of the previous types) is also supported as a single input. ",
-          1, type2char(anstype));
+  if (type0 != LGLSXP && type0 != INTSXP && type0 != REALSXP) {
+    error("Argument %d is of type %s. Only integer/logical and double types are supported. "
+          "A data.frame (of the previous types) is also supported as a single input. ",
+          1, type2char(type0));
   }
   for (int i = 1; i < n; ++i) {
     SEXPTYPE type = UTYPEOF(PTR_ETL(args, i));
     R_xlen_t len1 = xlength(PTR_ETL(args, i));
-    if (type != INTSXP && type != REALSXP) {
-      error("Argument %d is of type %s. Only integer and double types are supported.", i+1, type2char(type));
+    if (type != LGLSXP && type != INTSXP && type != REALSXP) {
+      error("Argument %d is of type %s. Only integer/logical and double types are supported.", i+1, type2char(type));
     }
     if (len1 != len0) {
       error("Argument %d is of length %zu but argument %d is of length %zu. "
@@ -434,8 +437,8 @@ SEXP pmeanR(SEXP na, SEXP args) {
   if (hasFactor) {
     error("Function 'pmean' is not meaningful for factors.");
   }
-  int nprotect=2;
-  SEXP ans = anstype != type0 ? PROTECT(coerceVector(args0, type0)) : PROTECT(duplicate(args0));
+  int nprotect = 2;
+  SEXP ans = type0 != REALSXP ? PROTECT(coerceVector(args0, REALSXP)) : PROTECT(duplicate(args0));
   const bool narm = asLogical(na);
   SEXP den = PROTECT(allocVector(REALSXP, len0));
   double *restrict pden = REAL(den);
@@ -454,8 +457,8 @@ SEXP pmeanR(SEXP na, SEXP args) {
   }
   for (int i = 1; i < n; ++i) {
     SEXPTYPE targsi = UTYPEOF(PTR_ETL(args, i));
-    if (targsi != type0) {
-      REPROTECT(dbl_a = coerceVector(PTR_ETL(args, i), type0), Idbl);
+    if (targsi != REALSXP) {
+      REPROTECT(dbl_a = coerceVector(PTR_ETL(args, i), REALSXP), Idbl);
     } else {
       REPROTECT(dbl_a = PTR_ETL(args, i), Idbl);
     }
@@ -679,10 +682,8 @@ SEXP pcountR(SEXP x, SEXP args) {
   return ans;
 }
 
-SEXP pcountNAR(SEXP x, SEXP args) {
-  if (xlength(x) != 1 || isNull(x)) {
-    error("Argument 'value' must be non NULL and length 1.");
-  }
+SEXP pcountNAR(SEXP args) {
+  
   const int n=length(args);
   if (n < 1) {
     error("Please supply at least 1 argument. (%d argument supplied)", n);
@@ -690,52 +691,16 @@ SEXP pcountNAR(SEXP x, SEXP args) {
   const SEXP args0 = PTR_ETL(args, 0);
   SEXPTYPE anstype = UTYPEOF(args0);
   const R_xlen_t len0 = xlength(args0);
-  if (anstype != LGLSXP && anstype != INTSXP && anstype != REALSXP &&
-      anstype != CPLXSXP && anstype != STRSXP) {
-    error("Argument %d is of type %s. Only logical, integer, double, complex and"
-            " character types are supported.", 1, type2char(anstype));
-  }
-  SEXPTYPE tx = UTYPEOF(x);
-  if (anstype != tx) {
-    error("Type of 'value' (%s) is different than type of Argument %d (%s). "
-            "Please make sure both have the same type.", type2char(tx), 1, type2char(anstype));
-  }
-  SEXP classx = PROTECT(getAttrib(x, R_ClassSymbol));
-  if(!R_compute_identical(PROTECT(getAttrib(args0, R_ClassSymbol)), classx, 0)) {
-    error("Class of 'value' is different than class of Argument %d. "
-            "Please make sure both have the same class.", 1);
-  }
-  UNPROTECT(1);
-  int nprotect = 0;
-  const bool xf = isFactor(x);
-  SEXP levelsx;
-  if (xf) {
-    levelsx = PROTECT(getAttrib(x, R_LevelsSymbol)); nprotect++;
-    if (!R_compute_identical(levelsx, PROTECT(getAttrib(args0, R_LevelsSymbol)), 0)) {
-      error("Levels of 'value' are different than levels of Argument %d. "
-              "Please make sure both have the same levels.", 1);
-    }
-    UNPROTECT(1);
+  if (!IS_VALID_TYPE(anstype)) {
+    error("Argument %d is of type %s. Only logical, integer, double, complex, "
+          "character and list types are supported.", 1, type2char(anstype));
   }
   for (int i = 1; i < n; ++i) {
     SEXPTYPE type = UTYPEOF(PTR_ETL(args, i));
     R_xlen_t len1 = xlength(PTR_ETL(args, i));
-    if (type != anstype) {
-      error("Type of argument %d is %s but argument %d is of type %s. "
-              "Please make sure both have the same type.", i+1,
-              type2char(type), 1, type2char(anstype));
-    }
-    if(!R_compute_identical(PROTECT(getAttrib(PTR_ETL(args, i), R_ClassSymbol)), classx, 0)) {
-      error("Class of 'value' is different than class of Argument %d. "
-              "Please make sure both have the same class.", i+1);
-    }
-    UNPROTECT(1);
-    if (xf) {
-      if (!R_compute_identical(levelsx, PROTECT(getAttrib(PTR_ETL(args, i), R_LevelsSymbol)), 0)) {
-        error("Levels of 'value' are different than levels of Argument %d. "
-                "Please make sure both have the same levels.", i + 1);
-      }
-      UNPROTECT(1);
+    if (!IS_VALID_TYPE(type)) {
+      error("Argument %d is of type %s. Only logical, integer, double, complex, "
+            "character and list types are supported.", i+1, type2char(type));
     }
     if (len1 != len0) {
       error("Argument %d is of length %zu but argument %d is of length %zu. "
@@ -743,123 +708,261 @@ SEXP pcountNAR(SEXP x, SEXP args) {
               "clear to the readers of your code.", i+1, len1, 1, len0);
     }
   }
-  UNPROTECT(1 + nprotect);
   SEXP ans;
   if (len0 > INT_MAX) {
     ans = PROTECT(allocVector(REALSXP, len0)); // # nocov start
     double *restrict pans = REAL(ans);
     memset(pans, 0, (unsigned)len0*sizeof(double));
-    switch(anstype) {
+    for (int i = 0; i < n; ++i) {
+    switch(UTYPEOF(PTR_ETL(args, i))) {
     case LGLSXP: {
-      for (int i = 0; i < n; ++i) {
-        int *pa = LOGICAL(PTR_ETL(args, i));
-        for (ssize_t j = 0; j < len0; ++j) {
-          if (pa[j] == NA_LOGICAL) {
-            pans[j]++;
-          }
+      int *pa = LOGICAL(PTR_ETL(args, i));
+      for (ssize_t j = 0; j < len0; ++j) {
+        if (pa[j] == NA_LOGICAL) {
+          pans[j]++;
         }
       }
     } break;
     case INTSXP: {
-      for (int i = 0; i < n; ++i) {
-        int *pa = INTEGER(PTR_ETL(args, i));
-        for (ssize_t j = 0; j < len0; ++j) {
-          if (pa[j] == NA_INTEGER) {
-            pans[j]++;
-          }
+      int *pa = INTEGER(PTR_ETL(args, i));
+      for (ssize_t j = 0; j < len0; ++j) {
+        if (pa[j] == NA_INTEGER) {
+          pans[j]++;
         }
       }
     } break;
     case REALSXP: {
-      for (int i = 0; i < n; ++i) {
-        double *pa = REAL(PTR_ETL(args, i));
-        for (ssize_t j = 0; j < len0; ++j) {
-          if (ISNAN(pa[j])) {
-            pans[j]++;
-          }
+      double *pa = REAL(PTR_ETL(args, i));
+      for (ssize_t j = 0; j < len0; ++j) {
+        if (ISNAN(pa[j])) {
+          pans[j]++;
         }
       }
     } break;
     case CPLXSXP: {
-      for (int i = 0; i < n; ++i) {
-        Rcomplex *pa = COMPLEX(PTR_ETL(args, i));
-        for (ssize_t j = 0; j < len0; ++j) {
-          if (ISNAN_COMPLEX(pa[j])) {
-            pans[j]++;
-          }
+      Rcomplex *pa = COMPLEX(PTR_ETL(args, i));
+      for (ssize_t j = 0; j < len0; ++j) {
+        if (ISNAN_COMPLEX(pa[j])) {
+          pans[j]++;
         }
       }
     } break;
     case STRSXP: {
-      for (int i = 0; i < n; ++i) {
-        const SEXP pa = PTR_ETL(args, i);
-        const SEXP *restrict px = STRING_PTR(pa);
-        for (ssize_t j = 0; j < len0; ++j) {
-          if (px[j] == NA_STRING) {
-            pans[j]++;
-          }
+      const SEXP pa = PTR_ETL(args, i);
+      const SEXP *restrict px = STRING_PTR(pa);
+      for (ssize_t j = 0; j < len0; ++j) {
+        if (px[j] == NA_STRING) {
+          pans[j]++;
+        }
+      }
+    } break;
+    case VECSXP: {
+      const SEXP pa = PTR_ETL(args, i);
+      const SEXP *restrict px = SEXPPTR_RO(pa);
+      for (ssize_t j = 0; j < len0; ++j) {
+        if (xlength(px[j]) == 0) {
+          pans[j]++;
         }
       }
     } break;
     } // # nocov end
+    }
   } else {
     ans = PROTECT(allocVector(INTSXP, len0));
     int *restrict pans = INTEGER(ans);
     memset(pans, 0, (unsigned)len0*sizeof(int));
-    switch(anstype) {
+    for (int i = 0; i < n; ++i) {
+    switch(UTYPEOF(PTR_ETL(args, i))) {
     case LGLSXP: {
-      for (int i = 0; i < n; ++i) {
-        int *pa = LOGICAL(PTR_ETL(args, i));
-        for (ssize_t j = 0; j < len0; ++j) {
-          if (pa[j] == NA_LOGICAL) {
-            pans[j]++;
-          }
+      int *pa = LOGICAL(PTR_ETL(args, i));
+      for (ssize_t j = 0; j < len0; ++j) {
+        if (pa[j] == NA_LOGICAL) {
+          pans[j]++;
         }
       }
     } break;
     case INTSXP: {
-      for (int i = 0; i < n; ++i) {
-        int *pa = INTEGER(PTR_ETL(args, i));
-        for (ssize_t j = 0; j < len0; ++j) {
-          if (pa[j] == NA_INTEGER) {
-            pans[j]++;
-          }
+      int *pa = INTEGER(PTR_ETL(args, i));
+      for (ssize_t j = 0; j < len0; ++j) {
+        if (pa[j] == NA_INTEGER) {
+          pans[j]++;
         }
       }
     } break;
     case REALSXP: {
-      for (int i = 0; i < n; ++i) {
-        double *pa = REAL(PTR_ETL(args, i));
-        for (ssize_t j = 0; j < len0; ++j) {
-          if (ISNAN(pa[j])) {
-            pans[j]++;
-          }
+      double *pa = REAL(PTR_ETL(args, i));
+      for (ssize_t j = 0; j < len0; ++j) {
+        if (ISNAN(pa[j])) {
+          pans[j]++;
         }
       }
     } break;
     case CPLXSXP: {
-      for (int i = 0; i < n; ++i) {
-        Rcomplex *pa = COMPLEX(PTR_ETL(args, i));
-        for (ssize_t j = 0; j < len0; ++j) {
-          if (ISNAN_COMPLEX(pa[j])) {
-            pans[j]++;
-          }
+      Rcomplex *pa = COMPLEX(PTR_ETL(args, i));
+      for (ssize_t j = 0; j < len0; ++j) {
+        if (ISNAN_COMPLEX(pa[j])) {
+          pans[j]++;
         }
       }
     } break;
     case STRSXP: {
-      for (int i = 0; i < n; ++i) {
-        const SEXP pa = PTR_ETL(args, i);
-        const SEXP *restrict px = STRING_PTR(pa);
-        for (ssize_t j = 0; j < len0; ++j) {
-          if (px[j] == NA_STRING) {
-            pans[j]++;
-          }
+      const SEXP pa = PTR_ETL(args, i);
+      const SEXP *restrict px = STRING_PTR(pa);
+      for (ssize_t j = 0; j < len0; ++j) {
+        if (px[j] == NA_STRING) {
+          pans[j]++;
         }
       }
     } break;
+    case VECSXP: {
+      const SEXP pa = PTR_ETL(args, i);
+      const SEXP *restrict px = SEXPPTR_RO(pa);
+      for (ssize_t j = 0; j < len0; ++j) {
+        if (xlength(px[j]) == 0) {
+          pans[j]++;
+        }
+      }
+    } break;
+    }
     }
   }
   UNPROTECT(1);
   return ans;
 }
+
+
+SEXP pfirstR(SEXP last, SEXP args) {
+  
+  if(!IS_BOOL(last)) error("Argument 'na.rm' must be TRUE or FALSE and length 1.");
+  
+  const int n = length(args);
+  if(n == 1) return args;
+  if(n < 1) error("Please supply at least 1 argument. (%d argument supplied)", n);
+  
+  int nprotect = 1;
+  if(asLogical(last)) {
+    SEXP argsrev = PROTECT(allocVector(VECSXP, n)); ++nprotect;
+    for(int i = 0; i < n; ++i) SET_VECTOR_ELT(argsrev, i, PTR_ETL(args, n-i-1));
+    args = argsrev;
+  }
+  
+  const SEXP args0 = PTR_ETL(args, 0);
+  SEXPTYPE anstype = UTYPEOF(args0);
+  SEXPTYPE type0 = anstype;
+  const R_xlen_t len0 = xlength(args0);
+  int hasFactor = isFactor(args0);
+  SEXP levels = hasFactor ? getAttrib(args0, R_LevelsSymbol) : R_NilValue;
+  
+  for (int i = 1; i < n; ++i) {
+    SEXP argsi = PTR_ETL(args, i);
+    SEXPTYPE type = UTYPEOF(argsi);
+    R_xlen_t len1 = xlength(argsi);
+    if (len1 != len0) {
+      error("Argument %d is of length %zu but argument %d is of length %zu. "
+              "If you wish to 'recycle' your argument, please use rep() to make this intent "
+              "clear to the readers of your code.", i+1, len1, 1, len0);
+    }
+    if (type != anstype) {
+      if((type == REALSXP || type == INTSXP || type == LGLSXP) && (anstype == REALSXP || anstype == INTSXP || anstype == LGLSXP)) {
+        if(type == REALSXP) anstype = REALSXP;
+        else if(anstype == LGLSXP) anstype = INTSXP;
+      } else error("All arguments need to have the same data type, except for numeric and logical types");
+    }
+    if(type == INTSXP) hasFactor += isFactor(argsi);
+    if(hasFactor) {
+      if(hasFactor != i + 1) error("If one argument is a factor, all arguments need to be factors");
+      if(!R_compute_identical(getAttrib(argsi, R_LevelsSymbol), levels, 0)) error("All factors need to have identical levels");
+    } 
+  }
+  
+  SEXP ans = (anstype != type0) ? PROTECT(coerceVector(args0, anstype)) : PROTECT(duplicate(args0));
+  
+  switch(anstype) {
+  case LGLSXP:
+  case INTSXP: {
+    int *restrict pans = INTEGER(ans);
+    for (int i = 1; i < n; ++i) {
+      const int *restrict pa = INTEGER(PTR_ETL(args, i));
+      ssize_t nna = 0;
+      for (ssize_t j = 0; j < len0; ++j) {
+        if(pans[j] == NA_INTEGER) {
+          if(pa[j] == NA_INTEGER) ++nna;
+          else pans[j] = pa[j];
+        }
+      }
+      if(nna == 0) break;
+    }
+  } break;
+  case REALSXP: {
+    double *restrict pans = REAL(ans);
+    for (int i = 1; i < n; ++i) {
+      SEXPTYPE targsi = UTYPEOF(PTR_ETL(args, i));
+      ssize_t nna = 0;
+      if(targsi == INTSXP || targsi == LGLSXP) {
+        const int *restrict pa = INTEGER(PTR_ETL(args, i));
+        for (ssize_t j = 0; j < len0; ++j) {
+          if(ISNAN(pans[j])) {
+            if(pa[j] == NA_INTEGER) ++nna;
+            else pans[j] = (double)pa[j];
+          }
+        }
+      } else {
+        const double *restrict pa = REAL(PTR_ETL(args, i));
+        for (ssize_t j = 0; j < len0; ++j) {
+          if(ISNAN(pans[j])) {
+            if(ISNAN(pa[j])) ++nna;
+            else pans[j] = pa[j];
+          }
+        }
+      }
+      if(nna == 0) break;
+    }
+  } break;
+  case CPLXSXP: {
+    Rcomplex *restrict pans = COMPLEX(ans);
+    for (int i = 1; i < n; ++i) {
+      const Rcomplex *restrict pa = COMPLEX(PTR_ETL(args, i));
+      ssize_t nna = 0;
+      for (ssize_t j = 0; j < len0; ++j) {
+        if(ISNAN_COMPLEX(pans[j])) {
+          if(ISNAN_COMPLEX(pa[j])) ++nna;
+          else pans[j] = pa[j];
+        }
+      }
+      if(nna == 0) break;
+    }
+  } break;
+  case STRSXP: {
+    SEXP *restrict pans = STRING_PTR(ans);
+    for (int i = 1; i < n; ++i) {
+      const SEXP *restrict pa = STRING_PTR(PTR_ETL(args, i));
+      ssize_t nna = 0;
+      for (ssize_t j = 0; j < len0; ++j) {
+        if(pans[j] == NA_STRING) {
+          if(pa[j] == NA_STRING) ++nna;
+          else pans[j] = pa[j];
+        }
+      }
+      if(nna == 0) break;
+    }
+  } break;
+  case VECSXP: {
+    SEXP *restrict pans = (SEXP *)DATAPTR_RO(ans);
+    for (int i = 1; i < n; ++i) {
+      const SEXP *restrict pa = SEXPPTR_RO(PTR_ETL(args, i));
+      ssize_t nna = 0;
+      for (ssize_t j = 0; j < len0; ++j) {
+        if(xlength(pans[j]) == 0) {
+          if(xlength(pa[j]) == 0) ++nna;
+          else pans[j] = pa[j];
+        }
+      }
+      if(nna == 0) break;
+    }
+  } break;
+  default: error("Unsupported data type: %s", type2char(anstype));
+  }
+  
+  UNPROTECT(nprotect);
+  return ans;
+}
